program mvdocrun;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  StrUtils,
  ShellAPI,
  Windows;

CONST
  mvdrC_maxfiles = 10;
  mvdrC_maxfilesstr = '10';
  mvdrC_Valid_IntChars = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0'];
  mvdrC_swname = 'MV Document Run (MVDOCRUN)';
  mvdrC_swshortname = 'mvdocrun';
  mvdrC_version = 'version 1.0.0';
  mvdrC_author = 'Mario A. Valdez-Ramirez (mario@mariovaldez.org)';
  mvdrC_CRdata = 'Copyright 2005-2006 by';
  mvdrC_license = 'This program is free software; you can redistribute it and/or modify'#13#10
    + 'it under the terms of the GNU General Public License as published by'#13#10
    + 'the Free Software Foundation; either version 2 of the License, or (at'#13#10
    + 'your option) any later version.'#13#10#13#10
    + 'This program is distributed in the hope that it will be useful, but'#13#10
    + 'WITHOUT ANY WARRANTY; without even the implied warranty of'#13#10
    + 'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU'#13#10
    + 'General Public License for more details.'#13#10#13#10
    + 'You should have received a copy of the GNU General Public License'#13#10
    + 'along with this program; if not, write to the Free Software'#13#10
    + 'Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307'#13#10
    + 'USA'#13#10;
  mvdrC_usage = mvdrC_swshortname + ' loads one or more documents or executables.'#13#10
    + 'Usage: ' + mvdrC_swshortname + ' [-v] [-l] [-q] [-a] [-dnn] file [file...]'#13#10
    + '"file" is the program(s) you want to run or the document(s) you want'#13#10
    + '       to open. You can specify up to ' + mvdrC_maxfilesstr + ' files.'#13#10
    + ' -l    display the license of ' + mvdrC_swshortname + ' and exit.'#13#10
    + ' -v    be verbose (default: no details).'#13#10
    + ' -q    be quiet (default: print summary).'#13#10
    + ' -a    invoke all files (default: invoke only one file).'#13#10
    + ' -dnn  wait "nn" number of seconds before starting.'#13#10#13#10
    + 'The exit code is equal to the number of successfully invoked files.'#13#10;
  mvdrC_separator = '=========================================================';

VAR
  mvdr_filename_list, mvdr_dirname_list : ARRAY [1..mvdrC_maxfiles] OF STRING;
  mvdr_exec_exitcode : LONGINT;                 { Exit code of invoked file }
  mvdr_file_count, mvdr_param_count : LONGINT;  { General counters }
  mvdr_param_str : STRING;                      { Parameter buffer }
  mvdr_total_proc : LONGINT;                    { Total files to invoke }
  mvdr_total_good : LONGINT;                    { Count of total files successfully invoked }
  mvdr_beverbose : BOOLEAN;                     { To be verbose or not }
  mvdr_bequiet : BOOLEAN;                       { To be quiet or not }
  mvdr_runall : BOOLEAN;                        { To run all files or not }
  mvdr_waittime : LONGINT;                      { Number of seconds to wait before starting }


{$R *.res}



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
FUNCTION FNmvdr_StrToInt (SourceValue : STRING) : LONGINT;
VAR
  CurrChar : WORD;
  TargetStr : STRING;
  TargetInt : LONGINT;
BEGIN
  TargetStr := '';
  SourceValue := TRIM (SourceValue);
  FOR CurrChar := 1 TO LENGTH (SourceValue) DO
    IF (SourceValue[CurrChar] IN mvdrC_Valid_IntChars) THEN TargetStr := TargetStr + SourceValue[CurrChar] ELSE BREAK;
  TRY
    TargetInt := STRTOINT (TargetStr);
  EXCEPT
    TargetInt := 0;
  END;
  FNmvdr_StrToInt := TargetInt;
END;



BEGIN
  { If no parameters are found, display help text. }
  IF (ParamCount = 0) THEN
    BEGIN
      WRITELN (mvdrC_separator);
      WRITELN (mvdrC_swname + ' ' + mvdrC_version);
      WRITELN (mvdrC_CRdata + ' ' + mvdrC_author);
      WRITELN (mvdrC_separator);
      WRITELN (mvdrC_usage);
      WRITELN ('Press <ENTER> key to exit...');
      READLN;
      ExitCode := 0;
      EXIT;
    END;
  { Look for known switches: help, license, verbose, quiet, run-all. }
  mvdr_beverbose := FALSE;
  mvdr_bequiet := FALSE;
  mvdr_runall := FALSE;
  mvdr_waittime := 0;
  FOR mvdr_file_count := 1 TO ParamCount DO
    BEGIN
      mvdr_param_str := UPPERCASE (TRIM (ParamStr (mvdr_file_count)));
      { Display help text. }
      IF ((mvdr_param_str = '-H') OR (mvdr_param_str = '/H') OR (mvdr_param_str = '/?')) THEN
        BEGIN
          WRITELN (mvdrC_separator);
          WRITELN (mvdrC_swname + ' ' + mvdrC_version);
          WRITELN (mvdrC_CRdata + ' ' + mvdrC_author);
          WRITELN (mvdrC_separator);
          WRITELN (mvdrC_usage);
          ExitCode := 0;
          EXIT;
        END
      { Display license text. }
      ELSE IF ((mvdr_param_str = '-L') OR (mvdr_param_str = '/L')) THEN
        BEGIN
          WRITELN (mvdrC_separator);
          WRITELN (mvdrC_swname + ' ' + mvdrC_version);
          WRITELN (mvdrC_CRdata + ' ' + mvdrC_author);
          WRITELN (mvdrC_separator);
          WRITELN (mvdrC_license);
          ExitCode := 0;
          EXIT;
        END
      { Enable verbose mode. }
      ELSE IF ((mvdr_param_str = '-V') OR (mvdr_param_str = '/V')) THEN
        BEGIN
          mvdr_beverbose := TRUE;
          mvdr_bequiet := FALSE;
        END
      { Enabled quiet mode. }
      ELSE IF ((mvdr_param_str = '-Q') OR (mvdr_param_str = '/Q')) THEN
        BEGIN
          mvdr_beverbose := FALSE;
          mvdr_bequiet := TRUE;
        END
      { Enabled run-all mode. }
      ELSE IF ((mvdr_param_str = '-A') OR (mvdr_param_str = '/A')) THEN
        BEGIN
          mvdr_runall := TRUE;
        END
      { Enabled run-all mode. }
      ELSE IF ((LEFTSTR (mvdr_param_str, 2) = '-D') OR (LEFTSTR (mvdr_param_str, 2) = '/D')) THEN
        BEGIN
          mvdr_waittime := FNmvdr_StrToInt (COPY (mvdr_param_str, 3, 3));
        END;
    END;
  { Load and normalize all filenames and directories. }
  mvdr_file_count := 0;
  FOR mvdr_param_count := 1 TO ParamCount DO
    BEGIN
      mvdr_param_str := TRIM (ParamStr (mvdr_param_count));
      IF ((mvdr_param_str[1] <> '-') AND (mvdr_param_str[1] <> '/') AND (mvdr_file_count < mvdrC_maxfiles)) THEN
        BEGIN
          INC (mvdr_file_count);
          mvdr_filename_list[mvdr_file_count] := ExpandFileName (mvdr_param_str);
          mvdr_dirname_list[mvdr_file_count] := ExtractFilePath (mvdr_filename_list[mvdr_file_count]);
        END;
    END;
  { Invoke each loaded filename. If verbose mode enabled, display the details. }
  { If run-all mode enabled, try all files, if not, try until one is successful. }
  mvdr_total_proc := mvdr_file_count;
  mvdr_total_good := 0;
  IF (mvdr_beverbose = TRUE) THEN WRITELN (mvdrC_separator);
  IF (mvdr_waittime > 0) THEN
    BEGIN
      IF (mvdr_bequiet = FALSE) THEN WRITELN ('Waiting ' + INTTOSTR (mvdr_waittime) + ' seconds.');
      SLEEP (mvdr_waittime * 1000);
    END;
  FOR mvdr_file_count := 1 TO mvdr_total_proc DO
    BEGIN
      IF (mvdr_filename_list[mvdr_file_count] <> '') THEN
        BEGIN
          IF (mvdr_beverbose = TRUE) THEN
            BEGIN
              WRITELN ('File: ' + mvdr_filename_list[mvdr_file_count]);
              WRITELN ('Dir: ' + mvdr_dirname_list[mvdr_file_count]);
            END;
          mvdr_exec_exitcode := ShellExecute (0, NIL, PCHAR (mvdr_filename_list[mvdr_file_count]), NIL, PCHAR (mvdr_dirname_list[mvdr_file_count]), SW_SHOWNORMAL);
          IF (mvdr_exec_exitcode < 32) THEN
            BEGIN
              IF (mvdr_beverbose = TRUE) THEN WRITELN ('Error! (Code = ' + INTTOSTR (mvdr_exec_exitcode) + ').'#13#10);
            END
          ELSE
            BEGIN
              IF (mvdr_beverbose = TRUE) THEN WRITELN ('Success! (Code = ' + INTTOSTR (mvdr_exec_exitcode) + ').'#13#10);
              INC (mvdr_total_good);
              IF (mvdr_runall = FALSE) THEN BREAK;
            END;
        END;
    END;
  { Display summary if quiet mode is disabled. }
  IF (mvdr_bequiet = FALSE) THEN
    BEGIN
      WRITELN (mvdrC_separator);
      WRITELN ('Total processed = ' + INTTOSTR (mvdr_total_proc) + '.');
      WRITELN ('Total executed = ' + INTTOSTR (mvdr_total_good) + '.');
    END;
  { Set exit code to the total files successfully invoked. }
  ExitCode := mvdr_total_good;
  EXIT;
END.


